@@ -1,4 +1,4 @@
-from fontTools.misc.py23 import *
+from fontTools.misc.textTools import Tag, bytesjoin
 from .DefaultTable import DefaultTable
 import sys
 import array
@@ -34,6 +34,7 @@ class BaseTTXConverter(DefaultTable):
 	"""
 
 	def decompile(self, data, font):
+		"""Create an object from the binary data. Called automatically on access."""
 		from . import otTables
 		reader = OTTableReader(data, tableTag=self.tableTag)
 		tableClass = getattr(otTables, self.tableTag)
@@ -41,26 +42,28 @@ class BaseTTXConverter(DefaultTable):
 		self.table.decompile(reader, font)
 
 	def compile(self, font):
-		""" Create a top-level OTTableWriter for the GPOS/GSUB table.
-			Call the compile method for the the table
-				for each 'converter' record in the table converter list
-					call converter's write method for each item in the value.
-						- For simple items, the write method adds a string to the
-						writer's self.items list.
-						- For Struct/Table/Subtable items, it add first adds new writer to the
-						to the writer's self.items, then calls the item's compile method.
-						This creates a tree of writers, rooted at the GUSB/GPOS writer, with
-						each writer representing a table, and the writer.items list containing
-						the child data strings and writers.
-			call the getAllData method
-				call _doneWriting, which removes duplicates
-				call _gatherTables. This traverses the tables, adding unique occurences to a flat list of tables
-				Traverse the flat list of tables, calling getDataLength on each to update their position
-				Traverse the flat list of tables again, calling getData each get the data in the table, now that
-				pos's and offset are known.
+		"""Compiles the table into binary. Called automatically on save."""
 
-				If a lookup subtable overflows an offset, we have to start all over.
-		"""
+		# General outline:
+		# Create a top-level OTTableWriter for the GPOS/GSUB table.
+		# 	Call the compile method for the the table
+		# 		for each 'converter' record in the table converter list
+		# 			call converter's write method for each item in the value.
+		# 				- For simple items, the write method adds a string to the
+		# 				writer's self.items list.
+		# 				- For Struct/Table/Subtable items, it add first adds new writer to the
+		# 				to the writer's self.items, then calls the item's compile method.
+		# 				This creates a tree of writers, rooted at the GUSB/GPOS writer, with
+		# 				each writer representing a table, and the writer.items list containing
+		# 				the child data strings and writers.
+		# 	call the getAllData method
+		# 		call _doneWriting, which removes duplicates
+		# 		call _gatherTables. This traverses the tables, adding unique occurences to a flat list of tables
+		# 		Traverse the flat list of tables, calling getDataLength on each to update their position
+		# 		Traverse the flat list of tables again, calling getData each get the data in the table, now that
+		# 		pos's and offset are known.
+
+		# 		If a lookup subtable overflows an offset, we have to start all over.
 		overflowRecord = None
 
 		while True:
@@ -106,6 +109,10 @@ class BaseTTXConverter(DefaultTable):
 		self.table.populateDefaults()
 
 
+# https://github.com/fonttools/fonttools/pull/2285#issuecomment-834652928
+assert len(struct.pack('i', 0)) == 4
+assert array.array('i').itemsize == 4, "Oops, file a bug against fonttools."
+
 class OTTableReader(object):
 
 	"""Helper class to retrieve data from an OpenType table."""
@@ -140,32 +147,43 @@ class OTTableReader(object):
 		value, = struct.unpack(f">{typecode}", self.data[pos:newpos])
 		self.pos = newpos
 		return value
-
-	def readUShort(self):
-		return self.readValue("H", staticSize=2)
-
 	def readArray(self, typecode, staticSize, count):
 		pos = self.pos
 		newpos = pos + count * staticSize
 		value = array.array(typecode, self.data[pos:newpos])
 		if sys.byteorder != "big": value.byteswap()
 		self.pos = newpos
-		return value
-
-	def readUShortArray(self, count):
-		return self.readArray("H", staticSize=2, count=count)
+		return value.tolist()
 
 	def readInt8(self):
 		return self.readValue("b", staticSize=1)
+	def readInt8Array(self, count):
+		return self.readArray("b", staticSize=1, count=count)
 
 	def readShort(self):
 		return self.readValue("h", staticSize=2)
+	def readShortArray(self, count):
+		return self.readArray("h", staticSize=2, count=count)
 
 	def readLong(self):
-		return self.readValue("l", staticSize=4)
+		return self.readValue("i", staticSize=4)
+	def readLongArray(self, count):
+		return self.readArray("i", staticSize=4, count=count)
 
 	def readUInt8(self):
 		return self.readValue("B", staticSize=1)
+	def readUInt8Array(self, count):
+		return self.readArray("B", staticSize=1, count=count)
+
+	def readUShort(self):
+		return self.readValue("H", staticSize=2)
+	def readUShortArray(self, count):
+		return self.readArray("H", staticSize=2, count=count)
+
+	def readULong(self):
+		return self.readValue("I", staticSize=4)
+	def readULongArray(self, count):
+		return self.readArray("I", staticSize=4, count=count)
 
 	def readUInt24(self):
 		pos = self.pos
@@ -173,9 +191,8 @@ class OTTableReader(object):
 		value, = struct.unpack(">l", b'\0'+self.data[pos:newpos])
 		self.pos = newpos
 		return value
-
-	def readULong(self):
-		return self.readValue("L", staticSize=4)
+	def readUInt24Array(self, count):
+		return [self.readUInt24() for _ in range(count)]
 
 	def readTag(self):
 		pos = self.pos
@@ -208,14 +225,24 @@ class OTTableWriter(object):
 
 	"""Helper class to gather and assemble data for OpenType tables."""
 
-	def __init__(self, localState=None, tableTag=None):
+	def __init__(self, localState=None, tableTag=None, offsetSize=2):
 		self.items = []
 		self.pos = None
 		self.localState = localState
 		self.tableTag = tableTag
-		self.longOffset = False
+		self.offsetSize = offsetSize
 		self.parent = None
 
+	# DEPRECATED: 'longOffset' is kept as a property for backward compat with old code.
+	# You should use 'offsetSize' instead (2, 3 or 4 bytes).
+	@property
+	def longOffset(self):
+		return self.offsetSize == 4
+
+	@longOffset.setter
+	def longOffset(self, value):
+		self.offsetSize = 4 if value else 2
+
 	def __setitem__(self, name, value):
 		state = self.localState.copy() if self.localState else dict()
 		state[name] = value
@@ -236,7 +263,7 @@ class OTTableWriter(object):
 			if hasattr(item, "getCountData"):
 				l += item.size
 			elif hasattr(item, "getData"):
-				l += 4 if item.longOffset else 2
+				l += item.offsetSize
 			else:
 				l = l + len(item)
 		return l
@@ -250,9 +277,9 @@ class OTTableWriter(object):
 			item = items[i]
 
 			if hasattr(item, "getData"):
-				if item.longOffset:
+				if item.offsetSize == 4:
 					items[i] = packULong(item.pos - pos)
-				else:
+				elif item.offsetSize == 2:
 					try:
 						items[i] = packUShort(item.pos - pos)
 					except struct.error:
@@ -260,6 +287,10 @@ class OTTableWriter(object):
 						overflowErrorRecord = self.getOverflowErrorRecord(item)
 
 						raise OTLOffsetOverflowError(overflowErrorRecord)
+				elif item.offsetSize == 3:
+					items[i] = packUInt24(item.pos - pos)
+				else:
+					raise ValueError(item.offsetSize)
 
 		return bytesjoin(items)
 
@@ -274,7 +305,7 @@ class OTTableWriter(object):
 	def __eq__(self, other):
 		if type(self) != type(other):
 			return NotImplemented
-		return self.longOffset == other.longOffset and self.items == other.items
+		return self.offsetSize == other.offsetSize and self.items == other.items
 
 	def _doneWriting(self, internedTables):
 		# Convert CountData references to data string items
@@ -302,6 +333,12 @@ class OTTableWriter(object):
 				items[i] = item.getCountData()
 			elif hasattr(item, "getData"):
 				item._doneWriting(internedTables)
+				# At this point, all subwriters are hashable based on their items.
+				# (See hash and comparison magic methods above.) So the ``setdefault``
+				# call here will return the first writer object we've seen with
+				# equal content, or store it in the dictionary if it's not been
+				# seen yet. We therefore replace the subwriter object with an equivalent
+				# object, which deduplicates the tree.
 				if not dontShare:
 					items[i] = item = internedTables.setdefault(item, item)
 		self.items = tuple(items)
@@ -330,13 +367,13 @@ class OTTableWriter(object):
 			tables, extTables, done = extTables, None, {}
 
 		# add Coverage table if it is sorted last.
-		sortCoverageLast = 0
+		sortCoverageLast = False
 		if hasattr(self, "sortCoverageLast"):
 			# Find coverage table
 			for i in range(numItems):
 				item = self.items[i]
-				if hasattr(item, "name") and (item.name == "Coverage"):
-					sortCoverageLast = 1
+				if getattr(item, 'name', None) == "Coverage":
+					sortCoverageLast = True
 					break
 			if id(item) not in done:
 				item._gatherTables(tables, extTables, done)
@@ -349,7 +386,7 @@ class OTTableWriter(object):
 			if not hasattr(item, "getData"):
 				continue
 
-			if sortCoverageLast and (i==1) and item.name == 'Coverage':
+			if sortCoverageLast and (i==1) and getattr(item, 'name', None) == 'Coverage':
 				# we've already 'gathered' it above
 				continue
 
@@ -395,8 +432,8 @@ class OTTableWriter(object):
 
 	# interface for gathering data, as used by table.compile()
 
-	def getSubWriter(self):
-		subwriter = self.__class__(self.localState, self.tableTag)
+	def getSubWriter(self, offsetSize=2):
+		subwriter = self.__class__(self.localState, self.tableTag, offsetSize=offsetSize)
 		subwriter.parent = self # because some subtables have idential values, we discard
 					# the duplicates under the getAllData method. Hence some
 					# subtable writers can have more than one parent writer.
@@ -405,33 +442,52 @@ class OTTableWriter(object):
 
 	def writeValue(self, typecode, value):
 		self.items.append(struct.pack(f">{typecode}", value))
-
-	def writeUShort(self, value):
-		assert 0 <= value < 0x10000, value
-		self.items.append(struct.pack(">H", value))
-
-	def writeShort(self, value):
-		assert -32768 <= value < 32768, value
-		self.items.append(struct.pack(">h", value))
-
-	def writeUInt8(self, value):
-		assert 0 <= value < 256, value
-		self.items.append(struct.pack(">B", value))
+	def writeArray(self, typecode, values):
+		a = array.array(typecode, values)
+		if sys.byteorder != "big": a.byteswap()
+		self.items.append(a.tobytes())
 
 	def writeInt8(self, value):
 		assert -128 <= value < 128, value
 		self.items.append(struct.pack(">b", value))
+	def writeInt8Array(self, values):
+		self.writeArray('b', values)
+
+	def writeShort(self, value):
+		assert -32768 <= value < 32768, value
+		self.items.append(struct.pack(">h", value))
+	def writeShortArray(self, values):
+		self.writeArray('h', values)
+
+	def writeLong(self, value):
+		self.items.append(struct.pack(">i", value))
+	def writeLongArray(self, values):
+		self.writeArray('i', values)
+
+	def writeUInt8(self, value):
+		assert 0 <= value < 256, value
+		self.items.append(struct.pack(">B", value))
+	def writeUInt8Array(self, values):
+		self.writeArray('B', values)
+
+	def writeUShort(self, value):
+		assert 0 <= value < 0x10000, value
+		self.items.append(struct.pack(">H", value))
+	def writeUShortArray(self, values):
+		self.writeArray('H', values)
+
+	def writeULong(self, value):
+		self.items.append(struct.pack(">I", value))
+	def writeULongArray(self, values):
+		self.writeArray('I', values)
 
 	def writeUInt24(self, value):
 		assert 0 <= value < 0x1000000, value
 		b = struct.pack(">L", value)
 		self.items.append(b[1:])
-
-	def writeLong(self, value):
-		self.items.append(struct.pack(">l", value))
-
-	def writeULong(self, value):
-		self.items.append(struct.pack(">L", value))
+	def writeUInt24Array(self, values):
+		for value in values:
+			self.writeUInt24(value)
 
 	def writeTag(self, tag):
 		tag = Tag(tag).tobytes()
@@ -518,7 +574,11 @@ def packUShort(value):
 
 def packULong(value):
 	assert 0 <= value < 0x100000000, value
-	return struct.pack(">L", value)
+	return struct.pack(">I", value)
+
+def packUInt24(value):
+	assert 0 <= value < 0x1000000, value
+	return struct.pack(">I", value)[1:]
 
 
 class BaseTable(object):
@@ -553,7 +613,7 @@ class BaseTable(object):
 			countValue = 1
 			if conv.repeat:
 				if conv.repeat in reader:
-					countValue = reader[conv.repeat]
+					countValue = reader[conv.repeat] + conv.aux
 				else:
 					return NotImplemented
 			totalSize += size * countValue
@@ -636,9 +696,15 @@ class BaseTable(object):
 
 	def compile(self, writer, font):
 		self.ensureDecompiled()
+		# TODO Following hack to be removed by rewriting how FormatSwitching tables
+		# are handled.
+		# https://github.com/fonttools/fonttools/pull/2238#issuecomment-805192631
 		if hasattr(self, 'preWrite'):
+			deleteFormat = not hasattr(self, 'Format')
 			table = self.preWrite(font)
+			deleteFormat = deleteFormat and hasattr(self, 'Format')
 		else:
+			deleteFormat = False
 			table = self.__dict__.copy()
 
 		# some count references may have been initialized in a custom preWrite; we set
@@ -674,14 +740,11 @@ class BaseTable(object):
 				else:
 					# conv.repeat is a propagated count
 					writer[conv.repeat].setValue(countValue)
-				values = value
-				for i, value in enumerate(values):
-					try:
-						conv.write(writer, font, table, value, i)
-					except Exception as e:
-						name = value.__class__.__name__ if value is not None else conv.name
-						e.args = e.args + (name+'['+str(i)+']',)
-						raise
+				try:
+					conv.writeArray(writer, font, table, value)
+				except Exception as e:
+					e.args = e.args + (conv.name+'[]',)
+					raise
 			elif conv.isCount:
 				# Special-case Count values.
 				# Assumption: a Count field will *always* precede
@@ -722,6 +785,9 @@ class BaseTable(object):
 				if conv.isPropagated:
 					writer[conv.name] = value
 
+		if deleteFormat:
+			del self.Format
+
 	def readFormat(self, reader):
 		pass
 
@@ -811,6 +877,26 @@ class FormatSwitchingBaseTable(BaseTable):
 		BaseTable.toXML(self, xmlWriter, font, attrs, name)
 
 
+class UInt8FormatSwitchingBaseTable(FormatSwitchingBaseTable):
+	def readFormat(self, reader):
+		self.Format = reader.readUInt8()
+
+	def writeFormat(self, writer):
+		writer.writeUInt8(self.Format)
+
+
+formatSwitchingBaseTables = {
+	"uint16": FormatSwitchingBaseTable,
+	"uint8": UInt8FormatSwitchingBaseTable,
+}
+
+def getFormatSwitchingBaseTableClass(formatType):
+	try:
+		return formatSwitchingBaseTables[formatType]
+	except KeyError:
+		raise TypeError(f"Unsupported format type: {formatType!r}")
+
+
 #
 # Support for ValueRecords
 #
@@ -923,6 +1009,13 @@ class ValueRecord(object):
 			format = format | valueRecordFormatDict[name][0]
 		return format
 
+	def getEffectiveFormat(self):
+		format = 0
+		for name,value in self.__dict__.items():
+			if value:
+				format = format | valueRecordFormatDict[name][0]
+		return format
+
 	def toXML(self, xmlWriter, font, valueName, attrs=None):
 		if attrs is None:
 			simpleItems = []
