@@ -204,7 +204,7 @@ class BuilderTest(object):
         assert builder.buildComponentRecord(None) is None
 
     def test_buildCoverage(self):
-        cov = builder.buildCoverage({"two", "four"}, {"two": 2, "four": 4})
+        cov = builder.buildCoverage(("two", "four", "two"), {"two": 2, "four": 4})
         assert getXML(cov.toXML) == [
             "<Coverage>",
             '  <Glyph value="two"/>',
@@ -839,8 +839,12 @@ class BuilderTest(object):
             "  <!-- Class2Count=3 -->",
             '  <Class1Record index="0">',
             '    <Class2Record index="0">',
+            '      <Value1 XPlacement="0" YPlacement="0"/>',
+            '      <Value2 XPlacement="0"/>',
             "    </Class2Record>",
             '    <Class2Record index="1">',
+            '      <Value1 XPlacement="0" YPlacement="0"/>',
+            '      <Value2 XPlacement="0"/>',
             "    </Class2Record>",
             '    <Class2Record index="2">',
             '      <Value1 XPlacement="-80" YPlacement="-20"/>',
@@ -849,12 +853,15 @@ class BuilderTest(object):
             "  </Class1Record>",
             '  <Class1Record index="1">',
             '    <Class2Record index="0">',
+            '      <Value1 XPlacement="0" YPlacement="0"/>',
+            '      <Value2 XPlacement="0"/>',
             "    </Class2Record>",
             '    <Class2Record index="1">',
+            '      <Value1 XPlacement="0" YPlacement="0"/>',
             '      <Value2 XPlacement="-20"/>',
             "    </Class2Record>",
             '    <Class2Record index="2">',
-            "      <Value1/>",
+            '      <Value1 XPlacement="0" YPlacement="0"/>',
             '      <Value2 XPlacement="-50"/>',
             "    </Class2Record>",
             "  </Class1Record>",
@@ -911,9 +918,11 @@ class BuilderTest(object):
                 ("A", "zero"): (d0, d50),
                 ("A", "one"): (None, d20),
                 ("B", "five"): (d8020, d50),
+
             },
             self.GLYPHMAP,
         )
+
         assert getXML(subtable.toXML) == [
             '<PairPos Format="1">',
             "  <Coverage>",
@@ -927,10 +936,12 @@ class BuilderTest(object):
             "    <!-- PairValueCount=2 -->",
             '    <PairValueRecord index="0">',
             '      <SecondGlyph value="zero"/>',
+            '      <Value1 XPlacement="0" YPlacement="0"/>',
             '      <Value2 XPlacement="-50"/>',
             "    </PairValueRecord>",
             '    <PairValueRecord index="1">',
             '      <SecondGlyph value="one"/>',
+            '      <Value1 XPlacement="0" YPlacement="0"/>',
             '      <Value2 XPlacement="-20"/>',
             "    </PairValueRecord>",
             "  </PairSet>",
@@ -1031,8 +1042,8 @@ class BuilderTest(object):
             "  </Coverage>",
             '  <ValueFormat value="3"/>',
             "  <!-- ValueCount=2 -->",
-            '  <Value index="0" XPlacement="777"/>',
-            '  <Value index="1" YPlacement="-888"/>',
+            '  <Value index="0" XPlacement="777" YPlacement="0"/>',
+            '  <Value index="1" XPlacement="0" YPlacement="-888"/>',
             "</SinglePos>",
         ]
 
@@ -1138,7 +1149,7 @@ buildStatTable_test_data = [
         '      </AxisValue>',
         '      <AxisValue index="1" Format="1">',
         '        <AxisIndex value="0"/>',
-        '        <Flags value="2"/>',
+        '        <Flags value="2"/>  <!-- ElidableAxisValueName -->',
         '        <ValueNameID value="256"/>  <!-- Regular -->',
         '        <Value value="400.0"/>',
         '      </AxisValue>',
@@ -1193,7 +1204,7 @@ buildStatTable_test_data = [
         '      </AxisValue>',
         '      <AxisValue index="1" Format="1">',
         '        <AxisIndex value="0"/>',
-        '        <Flags value="2"/>',
+        '        <Flags value="2"/>  <!-- ElidableAxisValueName -->',
         '        <ValueNameID value="258"/>  <!-- Regular -->',
         '        <Value value="400.0"/>',
         '      </AxisValue>',
@@ -1211,7 +1222,7 @@ buildStatTable_test_data = [
         '      </AxisValue>',
         '      <AxisValue index="4" Format="1">',
         '        <AxisIndex value="1"/>',
-        '        <Flags value="2"/>',
+        '        <Flags value="2"/>  <!-- ElidableAxisValueName -->',
         '        <ValueNameID value="258"/>  <!-- Regular -->',
         '        <Value value="100.0"/>',
         '      </AxisValue>',
@@ -1246,7 +1257,7 @@ buildStatTable_test_data = [
         '    <AxisValueArray>',
         '      <AxisValue index="0" Format="1">',
         '        <AxisIndex value="0"/>',
-        '        <Flags value="2"/>',
+        '        <Flags value="2"/>  <!-- ElidableAxisValueName -->',
         '        <ValueNameID value="257"/>  <!-- Regular -->',
         '        <Value value="400.0"/>',
         '      </AxisValue>',
@@ -1291,7 +1302,7 @@ buildStatTable_test_data = [
         '      </AxisValue>',
         '      <AxisValue index="1" Format="2">',
         '        <AxisIndex value="0"/>',
-        '        <Flags value="2"/>',
+        '        <Flags value="2"/>  <!-- ElidableAxisValueName -->',
         '        <ValueNameID value="258"/>  <!-- Text -->',
         '        <NominalValue value="14.0"/>',
         '        <RangeMinValue value="10.0"/>',
@@ -1354,7 +1365,7 @@ buildStatTable_test_data = [
         '      </AxisValue>',
         '      <AxisValue index="1" Format="1">',
         '        <AxisIndex value="1"/>',
-        '        <Flags value="2"/>',
+        '        <Flags value="2"/>  <!-- ElidableAxisValueName -->',
         '        <ValueNameID value="258"/>  <!-- Regular -->',
         '        <Value value="100.0"/>',
         '      </AxisValue>',
